@@ -10,7 +10,6 @@ from .dataframe import (
     filter_high_confidence_detections,
     get_dynamic_spark_schema,
     get_product_summary,
-    get_spark_schema,
     get_spark_schema_from_dataframe,
     ir_results_to_dataframe,
     ir_results_to_summary_dataframe,
@@ -23,7 +22,6 @@ __all__ = [
     "analyze_results_dataframe",
     "filter_high_confidence_detections",
     "get_product_summary",
-    "get_spark_schema",
     "get_spark_schema_from_dataframe",
     "get_dynamic_spark_schema",
     "to_spark_dataframe",
