@@ -26,7 +26,7 @@ if TYPE_CHECKING:
         StructType = None
 
 
-def get_dynamic_spark_schema(df: pd.DataFrame) -> 'StructType | None':
+def get_dynamic_spark_schema(df: pd.DataFrame) -> "StructType | None":
     """
     Dynamically generate a Spark schema based on the actual DataFrame structure.
 
@@ -65,28 +65,38 @@ def get_dynamic_spark_schema(df: pd.DataFrame) -> 'StructType | None':
 
     for column_name, dtype in df.dtypes.items():
         # Handle different pandas dtypes
-        if dtype == 'object':
+        if dtype == "object":
             # Check if it's a datetime column
-            if column_name in ['result_created_at', 'result_updated_at']:
+            if column_name in ["result_created_at", "result_updated_at"]:
                 spark_type = TimestampType()
             # Check if it's the alternative_predictions column (list of dicts)
-            elif column_name == 'alternative_predictions':
+            elif column_name == "alternative_predictions":
                 # Define schema for alternative prediction items
-                alt_pred_schema = SparkStructType([
-                    StructField("category_id", IntegerType(), True),
-                    StructField("category_name", StringType(), True),
-                    StructField("score", FloatType(), True),
-                ])
+                alt_pred_schema = SparkStructType(
+                    [
+                        StructField("category_id", IntegerType(), True),
+                        StructField("category_name", StringType(), True),
+                        StructField("score", FloatType(), True),
+                    ]
+                )
                 spark_type = ArrayType(alt_pred_schema)
+            elif column_name in ["is_beer", "orientation", "realogram_slot", "realogram_shelf", "realogram_stack"]:
+                spark_type = StringType()
+            elif column_name.startswith("share_"):
+                spark_type = FloatType()
             else:
                 spark_type = StringType()
-        elif dtype == 'int64':
+        elif dtype == "int64":
             spark_type = IntegerType()
-        elif dtype == 'float64':
-            spark_type = FloatType()
-        elif dtype == 'bool':
+        elif dtype == "float64":
+            # Handle score columns
+            if column_name.endswith("_score"):
+                spark_type = FloatType()
+            else:
+                spark_type = DoubleType()
+        elif dtype == "bool":
             spark_type = BooleanType()
-        elif dtype == 'datetime64[ns]':
+        elif dtype == "datetime64[ns]":
             spark_type = TimestampType()
         else:
             # Default to string for unknown types
@@ -97,7 +107,7 @@ def get_dynamic_spark_schema(df: pd.DataFrame) -> 'StructType | None':
     return StructType(fields)
 
 
-def get_spark_schema_from_dataframe(df: pd.DataFrame) -> 'StructType | None':
+def get_spark_schema_from_dataframe(df: pd.DataFrame) -> "StructType | None":
     """
     Generate Spark schema directly from the DataFrame structure.
 
@@ -119,82 +129,74 @@ def ir_results_to_dataframe(
     results: list[Any],
     include_bbox: bool = True,
     include_alternative_predictions: bool = True,
+    include_modalities: bool = True,
+    include_shares: bool = True,
 ) -> pd.DataFrame:
     """
     Convert a list of NLIRResult objects to a pandas DataFrame.
-
+    
     This function matches categories with annotations using the category_id
     and creates a flat DataFrame with all attributes for each detected item.
+    Optionally includes modalities and share of shelf data.
 
     Args:
         results: List of NLIRResult objects (from zia.models)
         include_bbox: Whether to include bounding box coordinates as separate columns
         include_alternative_predictions: Whether to include alternative predictions
+        include_modalities: Whether to include modalities data (is-beer, orientation, realogram)
+        include_shares: Whether to include share of shelf data
 
     Returns:
         pandas DataFrame with one row per detected item
-
-    Example:
-        >>> from zia.utils import ir_results_to_dataframe
-        >>> results = await client.image_recognition.get_all_task_results(task_uuid)
-        >>> df = ir_results_to_dataframe(results)
-        >>> print(df.head())
-
-    Raises:
-        ImportError: If pandas is not installed
     """
-    try:
-        import pandas as pd
-    except ImportError:
-        raise ImportError(
-            "pandas is required for DataFrame operations. "
-            "Install it with: pip install pandas"
-        )
-
     rows = []
+    
+    # Pre-process share data to create a lookup map
+    share_lookup = {}
+    if include_shares:
+        for result in results:
+            if result.postprocessing_results and result.postprocessing_results.shares:
+                for share in result.postprocessing_results.shares:
+                    for value in share.values:
+                        # Create key: (image_id, product_uuid)
+                        key = (share.image_id, value.product_uuid)
+                        share_lookup[key] = {
+                            "count": value.count,
+                            "count_ratio": value.count_ratio,
+                            "area": value.area,
+                            "area_ratio": value.area_ratio,
+                        }
 
     for result in results:
-        if not result.coco or result.status.value != "PROCESSED":
+        if not result.coco:
             continue
 
-        # Create a mapping of category_id to category for quick lookup
+        # Create category lookup map
         category_map = {cat.id: cat for cat in result.coco.categories}
 
         for annotation in result.coco.annotations:
-            # Get the corresponding category
             category = category_map.get(annotation.category_id)
             if not category:
                 continue
 
-            # Base row with result-level information
+            # Get product UUID from category
+            product_uuid = None
+            if category.neurolabs:
+                product_uuid = category.neurolabs.productUuid
+
             row = {
                 # Result-level information
                 "result_uuid": result.uuid,
                 "task_uuid": result.task_uuid,
+                "neurolabs_image_link": f"https://app.neurolabs.ai/task-management/{result.task_uuid}/result/{result.uuid}/validation",
                 "image_url": result.image_url,
                 "result_status": result.status.value,
                 "result_duration": result.duration,
                 "result_created_at": result.created_at,
                 "result_updated_at": result.updated_at,
-                "confidence_score": result.confidence_score,
+                "confidence_score": annotation.neurolabs.score if annotation.neurolabs else None, 
                 # Image information
                 "image_id": annotation.image_id,
-                "image_width": next(
-                    (
-                        img.width
-                        for img in result.coco.images
-                        if img.id == annotation.image_id
-                    ),
-                    None,
-                ),
-                "image_height": next(
-                    (
-                        img.height
-                        for img in result.coco.images
-                        if img.id == annotation.image_id
-                    ),
-                    None,
-                ),
                 "image_filename": next(
                     (
                         img.file_name
@@ -203,15 +205,20 @@ def ir_results_to_dataframe(
                     ),
                     None,
                 ),
+                # Category information
+                "category_id": annotation.category_id,
+                "category_name": category.name,
+                "product_uuid": product_uuid,
+                "product_name": category.neurolabs.name if category.neurolabs else None,
+                "product_brand": category.neurolabs.brand if category.neurolabs else None,
+                "product_barcode": category.neurolabs.barcode if category.neurolabs else None,
+                "product_custom_id": category.neurolabs.customId if category.neurolabs else None,
+                "product_label": category.neurolabs.label if category.neurolabs else None,
                 # Annotation information
                 "annotation_id": annotation.id,
-                "category_id": annotation.category_id,
+                "detection_score": annotation.neurolabs.score if annotation.neurolabs else None,
                 "area": annotation.area,
                 "iscrowd": annotation.iscrowd,
-                "detection_score": annotation.neurolabs.score,
-                # Category information
-                "category_name": category.name,
-                "category_supercategory": category.supercategory,
             }
 
             # Add bounding box coordinates if requested
@@ -225,18 +232,32 @@ def ir_results_to_dataframe(
                     }
                 )
 
-            # Add Neurolabs category information
-            if category.neurolabs:
-                row.update(
-                    {
-                        "product_uuid": category.neurolabs.productUuid,
-                        "product_name": category.neurolabs.name,
-                        "product_brand": category.neurolabs.brand,
-                        "product_barcode": category.neurolabs.barcode,
-                        "product_custom_id": category.neurolabs.customId,
-                        "product_label": category.neurolabs.label,
-                    }
-                )
+            # Add modalities data if requested and available
+            if include_modalities and annotation.neurolabs and annotation.neurolabs.modalities:
+                modalities = annotation.neurolabs.modalities
+                
+                # Extract is-beer modality
+                if modalities.is_beer and len(modalities.is_beer) > 0:
+                    row["is_beer"] = modalities.is_beer[0].value
+                    row["is_beer_score"] = modalities.is_beer[0].score
+
+                # Extract orientation modality
+                if modalities.orientation and len(modalities.orientation) > 0:
+                    row["orientation"] = modalities.orientation[0].value
+                    row["orientation_score"] = modalities.orientation[0].score
+
+                # Extract realogram modalities
+                if modalities.realogram_slot and len(modalities.realogram_slot) > 0:
+                    row["realogram_slot"] = modalities.realogram_slot[0].value
+                    row["realogram_slot_score"] = modalities.realogram_slot[0].score
+
+                if modalities.realogram_shelf and len(modalities.realogram_shelf) > 0:
+                    row["realogram_shelf"] = modalities.realogram_shelf[0].value
+                    row["realogram_shelf_score"] = modalities.realogram_shelf[0].score
+
+                if modalities.realogram_stack and len(modalities.realogram_stack) > 0:
+                    row["realogram_stack"] = modalities.realogram_stack[0].value
+                    row["realogram_stack_score"] = modalities.realogram_stack[0].score
 
             # Add alternative predictions if requested
             if (
@@ -256,14 +277,21 @@ def ir_results_to_dataframe(
                         }
                     )
                 row["alternative_predictions"] = alt_predictions
+            else:
+                # Ensure alternative_predictions is always a list for Spark compatibility
+                row["alternative_predictions"] = []
 
-            # Add modalities if present
-            if annotation.neurolabs.modalities:
-                for (
-                    modality_name,
-                    modality_value,
-                ) in annotation.neurolabs.modalities.items():
-                    row[f"modality_{modality_name}"] = modality_value
+            # Add share of shelf data if requested and available
+            if include_shares and product_uuid:
+                share_key = (annotation.image_id, product_uuid)
+                if share_key in share_lookup:
+                    share_data = share_lookup[share_key]
+                    row.update({
+                        "share_count": share_data["count"],
+                        "share_count_ratio": share_data["count_ratio"],
+                        "share_area": share_data["area"],
+                        "share_area_ratio": share_data["area_ratio"],
+                    })
 
             rows.append(row)
 
@@ -379,7 +407,9 @@ def analyze_results_dataframe(df: pd.DataFrame) -> dict[str, Any]:
 
     analysis = {
         "total_detections": len(df),
-        "unique_products": df["product_name"].nunique() if "product_name" in df.columns else 0,
+        "unique_products": df["product_name"].nunique()
+        if "product_name" in df.columns
+        else 0,
         "unique_images": df["result_uuid"].nunique(),
         "score_stats": {
             "mean": df["detection_score"].mean(),
@@ -387,7 +417,7 @@ def analyze_results_dataframe(df: pd.DataFrame) -> dict[str, Any]:
             "min": df["detection_score"].min(),
             "max": df["detection_score"].max(),
             "median": df["detection_score"].median(),
-        }
+        },
     }
 
     # Product analysis
@@ -402,7 +432,9 @@ def analyze_results_dataframe(df: pd.DataFrame) -> dict[str, Any]:
     # Score distribution
     score_ranges = {
         "high_confidence": len(df[df["detection_score"] >= 0.9]),
-        "medium_confidence": len(df[(df["detection_score"] >= 0.7) & (df["detection_score"] < 0.9)]),
+        "medium_confidence": len(
+            df[(df["detection_score"] >= 0.7) & (df["detection_score"] < 0.9)]
+        ),
         "low_confidence": len(df[df["detection_score"] < 0.7]),
     }
     analysis["score_distribution"] = score_ranges
@@ -415,10 +447,59 @@ def analyze_results_dataframe(df: pd.DataFrame) -> dict[str, Any]:
             "avg_area": df["area"].mean(),
         }
 
+    # Modalities analysis (if available)
+    modality_columns = ["is_beer", "orientation", "realogram_slot", "realogram_shelf", "realogram_stack"]
+    available_modalities = [col for col in modality_columns if col in df.columns]
+    
+    if available_modalities:
+        analysis["modalities"] = {}
+        for modality in available_modalities:
+            analysis["modalities"][modality] = df[modality].value_counts().to_dict()
+            
+            # Add score analysis if available
+            score_col = f"{modality}_score"
+            if score_col in df.columns:
+                analysis["modalities"][f"{modality}_score_stats"] = {
+                    "mean": df[score_col].mean(),
+                    "std": df[score_col].std(),
+                    "min": df[score_col].min(),
+                    "max": df[score_col].max(),
+                    "median": df[score_col].median(),
+                }
+
+    # Share of shelf analysis (if available)
+    share_columns = [col for col in df.columns if col.startswith("share_")]
+    if share_columns:
+        analysis["share_of_shelf"] = {
+            "total_share_entries": len(df[df["row_type"] == "share"]) if "row_type" in df.columns else 0,
+            "unique_share_products": df["share_product_uuid"].nunique() if "share_product_uuid" in df.columns else 0,
+        }
+        
+        # Share statistics
+        if "share_count_ratio" in df.columns:
+            analysis["share_of_shelf"]["count_ratio_stats"] = {
+                "mean": df["share_count_ratio"].mean(),
+                "std": df["share_count_ratio"].std(),
+                "min": df["share_count_ratio"].min(),
+                "max": df["share_count_ratio"].max(),
+                "median": df["share_count_ratio"].median(),
+            }
+        
+        if "share_area_ratio" in df.columns:
+            analysis["share_of_shelf"]["area_ratio_stats"] = {
+                "mean": df["share_area_ratio"].mean(),
+                "std": df["share_area_ratio"].std(),
+                "min": df["share_area_ratio"].min(),
+                "max": df["share_area_ratio"].max(),
+                "median": df["share_area_ratio"].median(),
+            }
+
     return analysis
 
 
-def filter_high_confidence_detections(df: pd.DataFrame, threshold: float = 0.9) -> pd.DataFrame:
+def filter_high_confidence_detections(
+    df: pd.DataFrame, threshold: float = 0.9
+) -> pd.DataFrame:
     """
     Filter DataFrame to include only high-confidence detections.
 
@@ -467,20 +548,41 @@ def get_product_summary(df: pd.DataFrame) -> pd.DataFrame:
     if df.empty:
         return pd.DataFrame()
 
-    product_summary = df.groupby([
-        "product_uuid", "product_name", "product_brand", "product_barcode", "product_custom_id"
-    ]).agg({
-        "result_uuid": "count",  # Number of detections
-        "detection_score": ["mean", "max", "min", "std"],
-        "task_uuid": "nunique",  # Number of tasks this product appears in
-        "result_uuid": "nunique",  # Number of images this product appears in
-    }).reset_index()
+    product_summary = (
+        df.groupby(
+            [
+                "product_uuid",
+                "product_name",
+                "product_brand",
+                "product_barcode",
+                "product_custom_id",
+            ]
+        )
+        .agg(
+            {
+                "result_uuid": "count",  # Number of detections
+                "detection_score": ["mean", "max", "min", "std"],
+                "task_uuid": "nunique",  # Number of tasks this product appears in
+                "result_uuid": "nunique",  # Number of images this product appears in
+            }
+        )
+        .reset_index()
+    )
 
     # Flatten column names
     product_summary.columns = [
-        "product_uuid", "product_name", "product_brand", "product_barcode", "product_custom_id",
-        "total_detections", "avg_score", "max_score", "min_score", "score_std",
-        "num_tasks", "num_images"
+        "product_uuid",
+        "product_name",
+        "product_brand",
+        "product_barcode",
+        "product_custom_id",
+        "total_detections",
+        "avg_score",
+        "max_score",
+        "min_score",
+        "score_std",
+        "num_tasks",
+        "num_images",
     ]
 
     return product_summary
@@ -491,6 +593,8 @@ def to_spark_dataframe(
     spark_session,
     include_bbox: bool = True,
     include_alternative_predictions: bool = True,
+    include_modalities: bool = True,
+    include_shares: bool = True,
 ):
     """
     Convert NLIRResult objects to a Spark DataFrame.
@@ -500,6 +604,8 @@ def to_spark_dataframe(
         spark_session: Active Spark session
         include_bbox: Whether to include bounding box fields
         include_alternative_predictions: Whether to include alternative predictions
+        include_modalities: Whether to include modalities data (is-beer, orientation, realogram)
+        include_shares: Whether to include share of shelf data
 
     Returns:
         Spark DataFrame
@@ -519,14 +625,27 @@ def to_spark_dataframe(
     pdf = ir_results_to_dataframe(
         results,
         include_bbox=include_bbox,
-        include_alternative_predictions=include_alternative_predictions
+        include_alternative_predictions=include_alternative_predictions,
+        include_modalities=include_modalities,
+        include_shares=include_shares,
     )
 
     if pdf.empty:
         return spark_session.createDataFrame([], spark_session.sparkContext.emptyRDD())
 
+    # Replace NaN values with None for Spark compatibility
+    pdf = pdf.where(pd.notnull(pdf), None)
+
     # Generate schema from the pandas DataFrame
     schema = get_spark_schema_from_dataframe(pdf)
 
-    # Convert to Spark DataFrame
-    return spark_session.createDataFrame(pdf, schema=schema)
+    # Convert to Spark DataFrame using records
+    try:
+        return spark_session.createDataFrame(pdf, schema=schema)
+    except Exception as e:
+        # Provide more detailed error information
+        print(f"Error converting to Spark DataFrame: {e}")
+        print(f"DataFrame shape: {pdf.shape}")
+        print(f"DataFrame columns: {pdf.columns.tolist()}")
+        print(f"DataFrame dtypes: {pdf.dtypes.to_dict()}")
+        raise
